/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Preferences Toolbar 3.
 *
 * The Initial Developer of the Original Code is
 * Manuel Reimer.
 * Portions created by the Initial Developer are Copyright (C) 2002
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s): Manuel Reimer <Manuel.Reimer@gmx.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


var goPrefBar = null;

const ImportType_Import = 1
const ImportType_Update = 2
const ImportType_Reset = 3

function Init(go) {
    goPrefBar = go;

    prefbarCheckForUpdate();
}

function Export(win, toexport, filename) {
    var RDF = goPrefBar.RDF.RDFService;
    var mDatasource = goPrefBar.RDF.mDatasource;

    if (toexport.length == 0) return;

    var fphandler = Components.classes["@mozilla.org/network/protocol;1?name=file"].getService(Components.interfaces.nsIFileProtocolHandler);
    var exportfileobj = fphandler.getFileFromURLSpec(filename);

    // If the file exists then empty it by deleting and recreating it.
    try {
	if (exportfileobj.exists())
	    exportfileobj.remove(0);
	exportfileobj.create(0x00,0644);
    }
    catch(e) {
	win.alert(goPrefBar.GetString("importexport.preferences", "exporterrreadonly"));
	return;
    }

    var expDatasource = RDF.GetDataSourceBlocking(filename);

    var gRDFCU = Components.classes['@mozilla.org/rdf/container-utils;1'].getService();
    gRDFCU = gRDFCU.QueryInterface(Components.interfaces.nsIRDFContainerUtils);

    var Rbuttons = RDF.GetResource("urn:prefbar:buttons");

    var Container = gRDFCU.MakeSeq(expDatasource, Rbuttons);

    for (var i = 0; i < toexport.length; i++) {
	var RitemId = RDF.GetResource(toexport[i]);

	Container.InsertElementAt(RitemId, 1, "true");

	var properties = mDatasource.ArcLabelsOut(RitemId);
	while(properties.hasMoreElements()) {
	    var Rproperty = properties.getNext();

	    var Nvalue = mDatasource.GetTarget(RitemId, Rproperty, true);

	    expDatasource.Assert(RitemId, Rproperty, Nvalue, true);
	}
    }

    var infoSection = RDF.GetResource("urn:prefbar:info");
    var fversionAttrib = RDF.GetResource("http://www.xulplanet.com/rdf/prefbar#formatversion");
    var newVersion = RDF.GetLiteral(goPrefBar.RDF.FormatVersion);
    expDatasource.Assert(infoSection, fversionAttrib, newVersion, true);
    
    var remoteControll = expDatasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource);
    remoteControll.Flush();

    RDF.UnregisterDataSource(expDatasource);


    //****HACK**** (Mozilla Bug 236920) Check if the file was saved on a disc which didn't have enough space free.
    //             Sometimes flush will fill the whole disc up to the last byte and sometimes it simply
    //             doesn't save anything. We check for both.
    if (exportfileobj.fileSize == 0) {
	win.alert(goPrefBar.GetString("importexport.preferences", "exporterrfailedwrite"));
	return;
    }

    var fileurlpath = filename.substr(0, filename.lastIndexOf("/") + 1);
    var temppathobj = fphandler.getFileFromURLSpec(fileurlpath);
    temppathobj.append(prefbarGetTemporaryFile(temppathobj, "prefbar", ".tmp"));
    var abyte = "x";
    try	{
	try {
	    temppathobj.create(0x00,0644);
	} catch (e) { }
	var stream = Components.classes['@mozilla.org/network/file-output-stream;1'].createInstance(Components.interfaces.nsIFileOutputStream);
	stream.init(temppathobj, 2, 0x200, false); // open as "write only"

	stream.write(abyte, abyte.length);
	stream.close();
    } catch (e) {
	win.alert(prefbarGetString("importexport.preferences", "exporterrfailedwrite"));
	temppathobj.remove(false);
	return;
    }
    temppathobj.remove(false);
    //****END HACK****
}

function Import(win, origfilename, ImportType, forceoverwrite) {
    var RDF = goPrefBar.RDF.RDFService;
    var mDatasource = goPrefBar.RDF.mDatasource;

    goPrefBar.dump("IMPORTER: Import started for " + origfilename);

    //****HACK**** (Mozilla Bug 237784) Copy to temporary file first
    // This hack only works if we have to handle the "file:"-protocol
    var filename = "";
    if(origfilename.toUpperCase().substr(0,5) == "FILE:") {
	var dirService = Components.classes["@mozilla.org/file/directory_service;1"].getService(Components.interfaces.nsIProperties);
	//var ioService = var service = Components.classes["@mozilla.org/network/io-service;1"].getService(Components.interfaces.nsIIOService);
	var fileprotocolService = Components.classes["@mozilla.org/network/protocol;1?name=file"].getService(Components.interfaces.nsIFileProtocolHandler);

	var tempDir = dirService.get("TmpD", Components.interfaces.nsIFile);
	var strtmpfilename = prefbarGetTemporaryFile(tempDir, "prefbar", ".rdf");

	var origfileobj = fileprotocolService.getFileFromURLSpec(origfilename);
	origfileobj.copyTo(tempDir, strtmpfilename);

	var tempfileobj = tempDir.clone();
	tempfileobj.append(strtmpfilename);

	filename = fileprotocolService.getURLSpecFromFile(tempfileobj);
    }
    else
        filename = origfilename;
    //****END HACK****

    var askedUser = false;
    var Overwrite = false;
	
    if (forceoverwrite != undefined)
    {
	Overwrite = forceoverwrite;
	askedUser = true;
    }
    if (ImportType == undefined) ImportType = ImportType_Import;

    var impDatasource = RDF.GetDataSourceBlocking(filename);

    if (!goPrefBar.RDF.CanReadFormat(impDatasource)) {
        win.alert(goPrefBar.GetString("importexport.preferences", "importerrversion").replace("$filename", origfilename));
        if(origfilename.toUpperCase().substr(0,5) == "FILE:")
            tempfileobj.remove(false);
        return false;
    }

    if (goPrefBar.RDF.FormatUpdateNeeded(impDatasource)) {
        /*
          We don't overwrite anything as long as we are forced to use a
          temporary file. Maybe this will change in future

        if (!win.confirm("The file \"" + origfilename + "\" had been created with an older PrefBar version. We have to update the format of that file before we are able to import. This will modify the file you want to import! Do you want to continue? (Fell free to backup the file before hitting \"Yes\").")) {
            if(origfilename.toUpperCase().substr(0,5) == "FILE:")
                tempfileobj.remove(false);
            return false;
        }
        */
        win.alert(goPrefBar.GetString("importexport.preferences", "importinfooldformat").replace("$filename", origfilename));
        goPrefBar.RDF.PerformFormatUpdates(impDatasource);
    }

    var RButtons = RDF.GetResource("urn:prefbar:buttons");

    var impbtncontainer = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
    impbtncontainer.Init(impDatasource, RButtons);
    var pbbtncontainer = Components.classes["@mozilla.org/rdf/container;1"].createInstance(Components.interfaces.nsIRDFContainer);
    pbbtncontainer.Init(mDatasource, RButtons);

    var impelements = impbtncontainer.GetElements();
    while (impelements.hasMoreElements())
    {
	var impelement = impelements.getNext();
	var strimpelement = impelement.QueryInterface(Components.interfaces.nsIRDFResource).Value;

	// TODO: Add security Check here

	var IdFree = !mDatasource.ArcLabelsOut(impelement).hasMoreElements();

	// Read the "dontupdatelistitems"-tag for the current item.
	var DontUpdateListItems = false;
	var DULattribute = RDF.GetResource("http://www.xulplanet.com/rdf/prefbar#dontupdatelistitems");
	var DULvalue = impDatasource.GetTarget(impelement, DULattribute, true);
	if (DULvalue)
	    DontUpdateListItems = DULvalue.QueryInterface(Components.interfaces.nsIRDFLiteral).Value;
	//alert(DontUpdateListItems);

	goPrefBar.dump("IMPORTER: -> Current Item is: " + strimpelement);
	goPrefBar.dump("IMPORTER: IdFree: " + IdFree);
		
	// If this item would overwrite existing Items, then
	if (!IdFree) {
	    // If the User wasn't asked before, then ask if he wants to
	    // overwrite this items
	    if(!askedUser) {
		Overwrite = win.confirm(goPrefBar.GetString("importexport.preferences", "importquestionoverwrite"));
		askedUser = true;
	    }
	    // If the user has chosen to not overwrite, then continue with the next
	    // Button
	    if(!Overwrite) continue;
	}

	// If the Id is not free, then remove all old properties
	if (!IdFree) {
	    //alert("killing Item: " + impelement.QueryInterface(Components.interfaces.nsIRDFResource).Value);
	    var oldproperties = mDatasource.ArcLabelsOut(impelement);

	    while(oldproperties.hasMoreElements()) {
		var Roldproperty = oldproperties.getNext();

		var OldPropertyString = Roldproperty.QueryInterface(Components.interfaces.nsIRDFResource).Value.toUpperCase();

		// Check if we are allowed to overwrite this property. If not, then continue
		if (!Import_AllowOverwrite(ImportType, OldPropertyString, DontUpdateListItems))
		    continue;

		var Loldvalue = mDatasource.GetTarget(impelement, Roldproperty, true);
		mDatasource.Unassert(impelement, Roldproperty, Loldvalue);
	    }

	    // Get sure that the existing item really has a place in the database index. If not, add it.
	    var curIndex = pbbtncontainer.IndexOf(impelement);
	    if (curIndex == -1) {
		goPrefBar.dump("IMPORTER: Repairing corrupted item: " + strimpelement);
		pbbtncontainer.InsertElementAt(impelement, 1, "true");
	    }
	}
	// If the Id doesn't exist, then create it
	else {
	    //alert("Inserting new item: " + impelement.QueryInterface(Components.interfaces.nsIRDFResource).Value);
	    pbbtncontainer.InsertElementAt(impelement, 1, "true");
	}

	//Add all new properties
	var impproperties = impDatasource.ArcLabelsOut(impelement);
	while(impproperties.hasMoreElements()) {
	    var Rimpproperty = impproperties.getNext();
			
	    var ImpPropertyString = Rimpproperty.QueryInterface(Components.interfaces.nsIRDFResource).Value.toUpperCase();
	    // If the current Item is not a new item (overwrites/updates existing one)
	    if (!IdFree) {
				
		// Check if we are allowed to overwrite this property (if not
                // then it hasn't been deleted before and still has the old
                // value). If not, then continue
		if (!Import_AllowOverwrite(ImportType, ImpPropertyString, DontUpdateListItems))
		    continue;
	    }
			
	    var Nvalue = impDatasource.GetTarget(impelement, Rimpproperty, true);

            //Nvalue = goPrefBar.RDF.removeOldNewlineHack(ImpPropertyString, Nvalue)

	    mDatasource.Assert(impelement, Rimpproperty, Nvalue, true);
	}
    }

    goPrefBar.RDF.mDatasource.QueryInterface(Components.interfaces.nsIRDFRemoteDataSource).Flush();

    //****HACK**** (Mozilla Bug 237784) Delete Temporary file
    // This hack only works if we have to handle the "file:"-protocol
    if(origfilename.toUpperCase().substr(0,5) == "FILE:")
        tempfileobj.remove(false);
    //****END HACK****
}

function Import_AllowOverwrite(ImportType, Property, option_DontUpdateListItems)
{
    // No need for additional checks if we reset. Everything has to be overwritten.
    if (ImportType == ImportType_Reset) return true;

    // **** LABEL PROPERTY ****
    if (Property == "HTTP://WWW.XULPLANET.COM/RDF/PREFBAR#LABEL") {
	// If we update, we don't touch the users labels
	if (ImportType == ImportType_Update)
	    return false;
    }

    // **** ENABLED PROPERTY ****
    if (Property == "HTTP://WWW.XULPLANET.COM/RDF/PREFBAR#ENABLED") {
	// We don't allow to modify the enabled state for all additional cases
	// (Import and Update).
	return false;
    }

    // **** OPTIONLABEL/OPTIONVALUE PROPERTY ****
    if (Property.match(/^HTTP:\/\/WWW.XULPLANET.COM\/RDF\/PREFBAR#OPTION(LABEL|VALUE)\d+/)) {
	// If we update, we don't touch listitems if the "DontUpdateListItems"-flag is "true"
	if (ImportType == ImportType_Update && option_DontUpdateListItems)
	    return false;
    }

    return true;
}

function prefbarGetTemporaryFile(Folder, Prefix, Suffix) {
    var tempfile = Components.classes["@mozilla.org/file/local;1"].createInstance(Components.interfaces.nsILocalFile);
    var counter = 1;
    var name = "";
    do {
	tempfile.initWithFile(Folder);
	name = Prefix + counter + Suffix;
	tempfile.append(name);
	counter++;
    } while(tempfile.exists())
	return name;
}

function prefbarCheckForUpdate() {
    var prefBranch = goPrefBar.PrefService.getBranch("");
    // Kill old Version property, if it exists
    if (prefBranch.prefHasUserValue("prefbar.version")) prefBranch.clearUserPref("prefbar.version");

    var savedVersion = navigator.preference("extensions.prefbar.version");
    if (savedVersion < goPrefBar.prefbarVersion || savedVersion == undefined) {
	var updatewanted = confirm(goPrefBar.GetString("importexport.preferences", "updatewanted"));
	if (updatewanted) {
	    Import(window, "chrome://prefbar/content/prefbar.rdf", ImportType_Update, true);
	    alert(goPrefBar.GetString("importexport.preferences", "updatefinished"));
	}
	
	prefBranch.setIntPref("extensions.prefbar.version", goPrefBar.prefbarVersion);
    }
}
